/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Component of: Red Hat Application Server
 * 
 * Initial Developers: Aizaz Ahmed
 *                     Vivek Lakshmanan
 *                     Andrew Overholt
 *                     Matthew Wringe
 *
 */

package olstore.action;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import olstore.form.CheckoutForm;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import olstore.session.ShoppingCartLocal;

public class CheckOutAction extends DemoBaseAction {
    
    /**
     * Acts as a relay for all item related tasks with a default action
     * 
     */
    public ActionForward execute ( ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        
        
        String submitType = ((CheckoutForm)form).getSubmitType();
        if ( submitType==null || submitType.equals(""))  {
            copyShoppingCartToForm  ( mapping, form, request, response );
        }else if(submitType.equals("update")){
            copyFormToShoppingCart(mapping, form, request, response);
            copyShoppingCartToForm  ( mapping, form, request, response );
        }else if( submitType.equals ( "submit" ) ) {
            copyFormToShoppingCart(mapping, form, request, response);
            copyShoppingCartToForm  ( mapping, form, request, response );
            resetForm(mapping, form, request, response);
            try{
                placeOrders ( mapping, form, request, response );
                ActionMessages messages = new ActionMessages();
                ActionMessage msg = new ActionMessage("checkout.save.success");
                messages.add("success", msg);
                saveMessages(request, messages);
            }catch(Exception e){
                ActionErrors errors = new ActionErrors();
                errors.add("error", new ActionError("errors.order.save", e.getMessage() ));
                saveErrors(request, errors);
                return (new ActionForward(mapping.getInput()));  
            }
        }
        
        return mapping.findForward ( "/checkout/updateCheckout" );
        
    }
    
    public void  resetForm ( ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        ((CheckoutForm) form).reset();
    }
    
    public void  copyShoppingCartToForm ( ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        HttpSession session = request.getSession(false);
        ShoppingCartLocal shoppingCart = (ShoppingCartLocal)session.getAttribute("shoppingCart");
        ArrayList cartEntries = shoppingCart.shoppingCartToDTOs();
        ((CheckoutForm)form).setCartEntries ( cartEntries );
        ((CheckoutForm)form).setTotalCost ( shoppingCart.getTotalCost());
    }
    
    public void  copyFormToShoppingCart ( ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        HttpSession session = request.getSession(false);
        ShoppingCartLocal shoppingCart = (ShoppingCartLocal) session.getAttribute("shoppingCart");
        shoppingCart.DTOsToShoppingCart(((CheckoutForm) form).getCartEntries());
    }
    
    
    
    /**
     */
    public void placeOrders ( ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        HttpSession session = request.getSession(false);
        ShoppingCartLocal shoppingCart = (ShoppingCartLocal) session.getAttribute("shoppingCart");
        shoppingCart.createOrders();
    }
    
    
    
    
}
