/**
 * Copyright (c) 2004 Red Hat, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Component of: Red Hat Application Server
 * 
 * Initial Developers: Aizaz Ahmed
 *                     Vivek Lakshmanan
 *                     Andrew Overholt
 *                     Matthew Wringe
 *
 */
package olstore.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import olstore.dto.UserValue;
import olstore.form.DemoDynaBaseForm;
import olstore.session.helper.UserHelper;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;


public class UserModAction extends DemoBaseAction {
    
    /**
     * Acts as a relay for all user modification related tasks with a default action
     *
     */
    public ActionForward execute ( ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception{
        
        try {
            DemoDynaBaseForm modUserForm=(DemoDynaBaseForm) form;
            String action = (String) modUserForm.get("submitType");
            
            if ( action.equals("new") ){
                loadUser (mapping, form, request, response);
                return mapping.findForward ("ModUser");
            }
            else if (action.equals("Cancel")){
                return new ActionForward("/views/index.do", true);
            }			
            else {
                loadUser (mapping, form, request, response);
                return mapping.findForward ("UpdateUser");
            }
            
            
        }
        catch ( Exception e ) {
            //Logging code here
            ActionErrors errors = new ActionErrors();
            errors.add("error", new ActionError("errors.item.load", e.getMessage() ));
            saveErrors(request, errors);
            // Return to same page
            return (new ActionForward(mapping.getInput()));
        }
        
    }
    
    /**
     * Load the form with the users data.
     */
    public void loadUser (ActionMapping mapping,
            ActionForm form,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        DemoDynaBaseForm modUserForm=(DemoDynaBaseForm) form;
        String username=request.getRemoteUser ();
        
        UserHelper userHelper = (UserHelper)getWebApplicationContext().getBean("userHelper");
        
        UserValue userValue=userHelper.getUserValue (username);
        
        //copy values from the DTO to the form
        BeanUtils.copyProperties (modUserForm, userValue);
        BeanUtils.copyProperties (modUserForm, userValue.getAddress());
    }
    
}



