/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer: Florent BENOIT
 * Contributor: Shenheng Liang
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.ant.jonasbase;

import java.io.File;

// use class defined in this package.
// Should use ant class but there is a problem
//import org.apache.tools.ant.taskdefs.Replace;

/**
 * Defines common replacement methods
 * @author Florent Benoit
 */
public class JReplace extends Replace implements BaseTaskItf {

    /**
     * configuration file used
     */
    private String configurationFile = null;

    /**
     * Information for the logger
     */
    private String logInfo = null;

    /**
     * JONAS_ROOT directory
     */
    private File jonasRoot = null;

    /**
     * Sets the configuration file
     * @param configurationFile The configurationFile to set.
     */
    public void setConfigurationFile(String configurationFile) {
        this.configurationFile = configurationFile;
    }

    /**
     * @param destDir The destDir to set.
     */
    public void setDestDir(File destDir) {
        setFile(new File(new File(destDir, "conf"), configurationFile));
    }

    /**
     * @param destFile The destination file to set.
     */
    public void setDestFile(File destFile) {
        setFile(destFile);
    }

    /**
     * Gets logger info (to be displayed)
     * @return logger info
     * @see org.objectweb.jonas.ant.jonasbase.BaseTaskItf#getLogInfo()
     */
    public String getLogInfo() {
        return logInfo;
    }

    /**
     * Set the info to be displayed by the logger
     * @param logInfo information to be displayed
     * @see org.objectweb.jonas.ant.jonasbase.BaseTaskItf#setLogInfo(java.lang.String)
     */
    public void setLogInfo(String logInfo) {
        this.logInfo = logInfo;
    }

    /**
     * @param jonasRoot The jonasRoot directory
     */
    public void setJonasRoot(File jonasRoot) {
        this.jonasRoot = jonasRoot;
    }

    /**
     * @return the jonasRoot.
     */
    protected File getJonasRoot() {
        return jonasRoot;
    }

}