/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jaxr;

import java.io.Serializable;
import java.util.Properties;

import javax.naming.BinaryRefAddr;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.Referenceable;

import org.objectweb.jonas.common.JNDIUtils;
import org.objectweb.jonas.jaxr.factory.JAXRFactory;


/**
 * Represents a JAXR Connectionfactory bound in Registry.
 * @author Guillaume Sauthier
 */
public class JAXRConnection implements Referenceable, Serializable {

    /**
     * Internal properties
     */
    private Properties p;

    /**
     * Constructs a JAXRConnection instance.
     * @param p properties used to configure connection
     */
    public JAXRConnection(Properties p) {
        this.p = p;
    }

    /**
     * @see javax.naming.Referenceable#getReference()
     */
    public Reference getReference() throws NamingException {

        // create new Reference using javax.xml.registry.ConnectionFactory Type
        // and our own Factory
        Reference ref = new Reference(JAXRFactory.FACTORY_TYPE, JAXRFactory.class.getName(), null);

        // store configuration properties
        ref.add(new BinaryRefAddr(JAXRFactory.PROPS_NAME, JNDIUtils.getBytesFromObject(p)));

        // return the instance
        return ref;
    }

}
