/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.mejb;

import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;

// JMX
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.NotificationFilter;

/**
 * Proxy object on client side allowing remote notification delivery.
 * Current implementation only supports one notification listener/listened managed object association
 * (suppose a removeNotificationListener is called before a subsequent addNotificationListener calls).
 * This is very a temporary solution.
 */
public class ListenerProxyImpl extends PortableRemoteObject implements ListenerProxy {

    NotificationListener listener = null;
    NotificationFilter filter = null;

    public ListenerProxyImpl() throws RemoteException {
        super();
    }    

    public void addNotificationListener(NotificationListener listener, NotificationFilter filter) 
        throws RemoteException 
    {
        //System.out.println("--------> IN ListenerProxyImpl.addNotificationListener");
        this.listener = listener;
        this.filter = filter;
    }
    public void removeNotificationListener(NotificationListener listener) 
        throws RemoteException 
    {
        if (listener.equals(this.listener)) {
            // reset proxy 
            this.listener = null;
            this.filter = null;          
        }
    }
    public void handleNotification(Notification notification, Object handback)
        throws RemoteException {
        //System.out.println("--------> In ListenerProxyImpl.handleNotification, handback = " + handback);
        listener.handleNotification(notification, handback);
    }
}
