/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Michel Bruno and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.resource;


import java.util.Properties;
import java.net.URL;

/**
 * MBean interface for general Resource Management
 * MBean type: Standard
 * MBean model: delegate
 *
 * @author Michel Bruno and Guillaume Riviere<br>
 * Contributor Michel-Ange Anton (add 'filename', 'inEarCase', 'earURL' properties)
 */
public interface JmxResourceAdapterMBean {

    /**
     * return the Adaptator Properties
     * @return Properties adaptor properties
     */
    Properties getProperties() ;

    /**
     * return the jndi name
     * @return String jndiname
     */
    String getJndiName() ;

    /**
     * Accessor the filename of the resource adapter.
     * @return String the filename
     */
    String getFilename();

    /**
     * Accessor the flag indicating if the resource adapter is in Ear.
     * @return boolean Flag if this resource adapter is in Ear
     */
    boolean isInEarCase();

    /**
     * Accessor the URL of the Ear if the resource adapter is in Ear.
     * @return URL of the Ear or null
     */
    URL getEarURL();
}
