/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 *
 */

package org.objectweb.jonas.security.interceptors.jeremie;

import org.objectweb.security.propagation.SecurityReceiver;
import org.objectweb.security.context.SecurityCurrent;
import org.objectweb.security.context.SecurityContext;

/**
 * This class represents the implementation of a SecurityReceiver in JOnAS.
 * @author Jeff Mesnil
 */
public class JonasSecurityReceiver implements SecurityReceiver {

    /**
     * Empty constructor
     */
    public JonasSecurityReceiver() {
    }

    public void received_request(int request_id, SecurityContext ctx) {
        // Gets SecurityCurrent object (always existing in JOnAS Server)
        SecurityCurrent current = SecurityCurrent.getCurrent();
        if (current == null) {
            return;
        }
        // Associates the current thread with this security context
        current.setSecurityContext(ctx);
    }

    public SecurityContext sending_reply(int request_id) {
        // Gets Current object (always existing in JOnAS Server)
        SecurityCurrent current = SecurityCurrent.getCurrent();
        if (current == null) {
            return null;
        }
        // Get the Security Context
        SecurityContext ctx = current.getSecurityContext();
        // associates the current thread with a new default SecurityContext
        current.setSecurityContext(new SecurityContext());
        //System.out.println ("JOnAS send a reply : "+
        // ctx.getCallerPrincipal().getName()) ;
        return ctx;
    }
}