/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Politecnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 *
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas_ejb.container;

import org.objectweb.carol.cmi.configuration.TraceCmi;
import org.objectweb.carol.cmi.ha.RequestId;
import org.objectweb.jonas.ha.interceptor.HACurrentDelegateImpl;

/**
 * Generic part of the EJBObject implementation for replicated SFSBs
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 */
public abstract class JRepEntityLocal extends JEntityLocal {

    /**
     * Constructor
     * @param bf
     */
    public JRepEntityLocal(JEntityFactory bf) {
        super(bf);
    }

    /* (non-Javadoc)
     * @see org.objectweb.jonas_ejb.container.JEntityLocal#postInvoke(org.objectweb.jonas_ejb.container.RequestCtx)
     */
    public void postInvoke(RequestCtx rctx, JEntityContext bctx) {
        try {
            postInvokeHook(bctx);
        } catch (Exception e) {
            e.printStackTrace();
            TraceCmi.error("Error calling postInvokeHook in sfsb.", e);
        } finally {
            super.postInvoke(rctx);
        }
    }

    /**
     * Implements the hook to perform the required replication
     * tasks after the bean context has been established
     * @param rctx The RequestCtx that was returned at preInvoke()
     */
    private void postInvokeHook(JEntityContext bctx) {
        if (TraceCmi.isDebugCmiHA()) {
            TraceCmi.debugCmiHA("---------------------------------------");
            TraceCmi.debugCmiHA("In PostInvoke hook for: " + bctx.getPrimaryKey());
        }

        // Create the rootId from the current HACtx/Thread association
        HACurrentDelegateImpl current = HACurrentDelegateImpl.getCurrent();
        // Obtain the root request
        RequestId rootId = null;
        try { 
            rootId = (RequestId) current.getRequests().get(0);
            JRepUtil.addEntityBean(bctx, rootId);
        } catch (ArrayIndexOutOfBoundsException e) {
            TraceCmi.debugCmiHA("Don't need to be replicate.");
        }
        
        if (TraceCmi.isDebugCmiHA()) {
            TraceCmi.debugCmiHA("---------------------------------------");
        }        
    }
}
