/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Polit??cnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 *
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.rmi.NoSuchObjectException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import org.objectweb.carol.cmi.ServerConfigException;
import org.objectweb.carol.cmi.configuration.TraceCmi;
import org.objectweb.jonas_ejb.deployment.api.SessionDesc;

/**
 * This class is the Standard Home for replicated SSB objects It exists only
 * for beans that have declared a Remote Interface.
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 * @author Benoit Pelletier  
 */
public abstract class JRepStatelessHome extends JSessionHome {

    /** 
     * Constructor
     * @param dd
     * @param bf
     * @throws RemoteException
     */
    public JRepStatelessHome(SessionDesc dd, JSessionFactory bf) throws RemoteException {
        super(dd, bf);
        if (TraceCmi.isDebugCmiHA()) {
            TraceCmi.debugCmiHA("JRepStatelessHome");
        }
    }

    /**
     * Get the remote object as a stub object for HA remote object LB
     * @param remote remote object
     * @return get remote object
     * @throws RemoteException
     * @throws ServerConfigException
     * @throws NoSuchObjectException if the clustered object is not found in the
     *         CMI registry
     */
    public Remote getHARemoteStub(Remote remote) throws RemoteException {
        return JRepUtil.getHARemoteStub(remote, dd);
    }
    
}
