/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.beans.relation.mou;

import java.rmi.RemoteException;

import javax.ejb.EJBObject;
import javax.ejb.FinderException;

/**
 * @author S.Chassande-Barrioz
 */
public interface ARemote extends EJBObject {

    String getId() throws RemoteException;

    /**
     * It replaces the referenced elements of the relation by the element of the
     * collection specified.
     * This method has the transactional attribut TX_SUPPORTS.
     * @param c is a collection of the primary key of the bean 'B'. Then the
     * elements are java.lang.String objects.
     * @throw a FinderException if among the collection a primary key does not
     * match to a bean.
     */
    void assignB(String c) throws FinderException, RemoteException;

    /**
     * It replaces the referenced elements of the relation by the element of the
     * collection specified.
     * This method has the transactional attribut TX_REQUIRES_NEW.
     * @param c is a collection of the primary key of the bean 'B'. Then the
     * elements are java.lang.String objects.
     * @throw a FinderException if among the collection a primary key does not
     * match to a bean.
     */
    void assignBInNewTx(String c) throws FinderException, RemoteException;


    /**
     * It returns the primary key of the referenced elements.
     * This method has the transactional attribut TX_SUPPORTS.
     * @return a collection of the primary key of the bean 'B'. Then the
     * elements are java.lang.String objects.
     */
    String retrieveB() throws RemoteException;

    /**
     * It returns the primary key of the referenced elements.
     * This method has the transactional attribut TX_REQUIRES_NEW.
     * @return a collection of the primary key of the bean 'B'. Then the
     * elements are java.lang.String objects.
     */
    String retrieveBInNewTx() throws RemoteException;



}
