/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.beans.worker;

import java.rmi.RemoteException;
import javax.ejb.RemoveException;
import javax.ejb.EJBObject;

/**
 * Worker remote interface
 * @author Philippe Durieux
 */
public interface Worker extends EJBObject {

    /**
     * set the count
     */
    public void setwcount(int c) throws RemoteException;

    /**
     * get the count
     */
    public int getwcount() throws RemoteException;

    /**
     * get the notify accepted count
     */
    public int getNotifyAccepted() throws RemoteException;

    /**
     * get the notify started count
     */
    public int getNotifyStarted() throws RemoteException;

    /**
     * get the notify rejected count
     */
    public int getNotifyRejected() throws RemoteException;

    /**
     * get the notify completed count
     */
    public int getNotifyCompleted() throws RemoteException;

    /**
     * Run n works, synchronously.
     */
    public void doWorks(int n) throws RemoteException;

    /**
     * Start n works
     */
    public void startWorks(int n) throws RemoteException;

    /**
     * Schedule n works
     * Notifications are sent (a WorkListner is used)
     */
    public void scheduleWorks(int n, long timeout) throws RemoteException;

    /**
     * Run 1 work synchronously in a TX
     */
    public void doWorkInTx() throws RemoteException;

    /**
     * Start 1 work in a TX
     */
    public void startWorkInTx() throws RemoteException;

    /**
     * Schedule 1 work in a TX
     */
    public void scheduleWorkInTx() throws RemoteException;
}
