/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.distribution;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.relation.omb.AHomeRemote;
import org.objectweb.jonas.jtests.beans.relation.omb.BHomeRemote;
import org.objectweb.jonas.jtests.beans.relation.omb.BRemote;
import org.objectweb.jonas.jtests.beans.relation.omb.Front;
import org.objectweb.jonas.jtests.beans.relation.omb.FrontHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * Test a session bean remote accessing CMP2 entities local, with
 * relationships.
 * Beans used: relation/omb
 * @author Philippe Durieux
 */
public class F_FrontalCMP2 extends JTestCase {

    private static String BEAN_HOME_A = "relation_omb_AHome";
    protected static AHomeRemote ahome = null;
    private static String BEAN_HOME_B = "relation_omb_BHome";
    protected static BHomeRemote bhome = null;
    protected static FrontHome fhome = null;

    public F_FrontalCMP2(String name) {
        super(name);
    }
    

    protected void setUp() {
        super.setUp();
        if (fhome == null) {
            useBeans("omb", false);
            try {
                fhome = (FrontHome) PortableRemoteObject.narrow(ictx.lookup("relation_omb_FrontHome"), FrontHome.class);
                ahome = (AHomeRemote) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_A), AHomeRemote.class);
                bhome = (BHomeRemote) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_B), BHomeRemote.class);
                assertNotNull(fhome);
                assertNotNull(ahome);
                assertNotNull(bhome);
            } catch (NamingException e) {
                fail("Cannot get bean home");
            }
        }
    }

    /**
     * Uses a session bean (inside ejbCreate) to create other beans
     * and find them in the same transaction.
     * This mimics a part of code of Bonita.
     */
    public void testCreateFind() throws Exception {
        // actual test
        Front front = fhome.create("project", "role");
        // cleaning part
        BRemote br = bhome.findByName("role", "project");
        br.remove();
        ahome.remove("project");
        front.remove();
    }

    /**
     * Test for stateful session bean serialization
     */
    public void testSerialization() throws Exception {
        final int tnb1 = 8;
        Front [] tarray1 = new Front[tnb1];
        for (int i = 0; i < tnb1; i++) {
            tarray1[i] = fhome.create("seri" + i, "num" + i);
        }
        for (int i = 0; i < tnb1; i++) {
            tarray1[i].remove();
        }
    }


    public static Test suite() {
        return new TestSuite(F_FrontalCMP2.class);
    }

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_FrontalCMP2(testtorun));
        }
    }
}
