/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.oboolean.Oboolean;
import org.objectweb.jonas.jtests.beans.etype.oboolean.ObooleanHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Boolean field of entity bean .
 * Beans used: etype/oboolean
 * @author Helene Joanin
 */
public abstract class A_EtypeOboolean extends JTestCase {

    public A_EtypeOboolean(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
    }

    /**
     * Return ObooleanHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public ObooleanHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjBooleanFindByPk() throws Exception {
        String pk = "pk1";
        Oboolean bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjBooleanFindByF1() throws Exception {
        boolean f1 = true;
        Collection cBeans = getHome().findByF1(new Boolean(f1));
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Oboolean bean = (Oboolean) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                             Oboolean.class);
            assertEquals("F1", f1, bean.getF1().booleanValue());
        }
    }

    /**
     * getF1() test
     */
    public void testObjBooleanGetF1() throws Exception {
        String pk = "pk2";
        Oboolean bean = getHome().findByPrimaryKey(pk);
        Boolean f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", true, f1.booleanValue());
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjBooleanGetF1Null() throws Exception {
        String pk = "pknull";
        Oboolean bean = getHome().findByPrimaryKey(pk);
        Boolean f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjBooleanSetF1() throws Exception {
        String pk = "pk3";
        Oboolean bean = getHome().findByPrimaryKey(pk);
        boolean f1 = true;
        bean.setF1(new Boolean(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).booleanValue());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjBooleanSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Oboolean bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjBooleanCreate() throws Exception {
        String pk = "pkcreated";
        Boolean f1 = new Boolean(true);
        Oboolean bean = getHome().create(pk, f1);
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, bean.getF1());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjBooleanRemove() throws Exception {
        String pk = "pktoremove";
        Oboolean bean = getHome().findByPrimaryKey(pk);
        Boolean f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
