/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.obyte.Obyte;
import org.objectweb.jonas.jtests.beans.etype.obyte.ObyteHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Byte field of entity bean .
 * Beans used: etype/obyte
 * @author Helene Joanin
 */
public abstract class A_EtypeObyte extends JTestCase {

    public A_EtypeObyte(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("obyte", true);
    }

    /**
     * Return ObyteHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public ObyteHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjByteFindByPk() throws Exception {
        String pk = "pk1";
        Obyte bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjByteFindByF1() throws Exception {
        byte f1 = (byte) 5;
        Collection cBeans = getHome().findByF1(new Byte(f1));
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Obyte bean = (Obyte) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                         Obyte.class);
            assertEquals("F1", f1, bean.getF1().byteValue());
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testObjByteGetF1() throws Exception {
        String pk = "pk2";
        Obyte bean = getHome().findByPrimaryKey(pk);
        Byte f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", 2, f1.byteValue());
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjByteGetF1Null() throws Exception {
        String pk = "pknull";
        Obyte bean = getHome().findByPrimaryKey(pk);
        Byte f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjByteSetF1() throws Exception {
        String pk = "pk3";
        Obyte bean = getHome().findByPrimaryKey(pk);
        byte f1 = (byte) 30;
        bean.setF1(new Byte(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).byteValue());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjByteSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Obyte bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjByteCreate() throws Exception {
        String pk = "pkcreated";
        byte f1 = (byte) 59;
        Obyte bean = getHome().create(pk, new Byte(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).byteValue());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjByteRemove() throws Exception {
        String pk = "pktoremove";
        Obyte bean = getHome().findByPrimaryKey(pk);
        Byte f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
