/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.odouble.Odouble;
import org.objectweb.jonas.jtests.beans.etype.odouble.OdoubleHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Double field of entity bean .
 * Beans used: etype/odouble
 * @author Helene Joanin
 */
public abstract class A_EtypeOdouble extends JTestCase {

    private static final double DELTA = 0.01d;

    public A_EtypeOdouble(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("odouble", true);
    }

    /**
     * Return OdoubleHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OdoubleHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjDoubleFindByPk() throws Exception {
        String pk = "pk1";
        Odouble bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjDoubleFindByF1() throws Exception {
        double f1 = 5.0d;
        Collection cBeans = getHome().findByF1(new Double(f1 - DELTA), new Double(f1 + DELTA));
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Odouble bean = (Odouble) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                           Odouble.class);
            assertEquals("F1", f1, (bean.getF1()).doubleValue(), DELTA);
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testObjDoubleGetF1() throws Exception {
        String pk = "pk2";
        Odouble bean = getHome().findByPrimaryKey(pk);
        Double f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", 2.0d, f1.doubleValue(), DELTA);
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjDoubleGetF1Null() throws Exception {
        String pk = "pknull";
        Odouble bean = getHome().findByPrimaryKey(pk);
        Double f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjDoubleSetF1() throws Exception {
        String pk = "pk3";
        Odouble bean = getHome().findByPrimaryKey(pk);
        double f1 = 30.0d;
        bean.setF1(new Double(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).doubleValue(), DELTA);
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjDoubleSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Odouble bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjDoubleCreate() throws Exception {
        String pk = "pkcreated";
        double f1 = 1959.65d;
        Odouble bean = getHome().create(pk, new Double(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).doubleValue(), DELTA);
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjDoubleRemove() throws Exception {
        String pk = "pktoremove";
        Odouble bean = getHome().findByPrimaryKey(pk);
        Double f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
