/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.ofloat.Ofloat;
import org.objectweb.jonas.jtests.beans.etype.ofloat.OfloatHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Float field of entity bean .
 * Beans used: etype/ofloat
 * @author Helene Joanin
 */
public abstract class A_EtypeOfloat extends JTestCase {

    private static final float DELTA = 0.01f;

    public A_EtypeOfloat(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("ofloat", true);
    }

    /**
     * Return OfloatHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OfloatHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjFloatFindByPk() throws Exception {
        String pk = "pk1";
        Ofloat bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjFloatFindByF1() throws Exception {
        float f1 = 5.0f;
        Collection cBeans = getHome().findByF1(new Float(f1 - DELTA), new Float(f1 + DELTA));
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Ofloat bean = (Ofloat) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                         Ofloat.class);
            assertEquals("F1", f1, (bean.getF1()).floatValue(), DELTA);
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testObjFloatGetF1() throws Exception {
        String pk = "pk2";
        Ofloat bean = getHome().findByPrimaryKey(pk);
        Float f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", 2.0, f1.floatValue(), DELTA);
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjFloatGetF1Null() throws Exception {
        String pk = "pknull";
        Ofloat bean = getHome().findByPrimaryKey(pk);
        Float f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjFloatSetF1() throws Exception {
        String pk = "pk3";
        Ofloat bean = getHome().findByPrimaryKey(pk);
        float f1 = 30.0f;
        bean.setF1(new Float(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).floatValue(), DELTA);
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjFloatSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Ofloat bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjFloatCreate() throws Exception {
        String pk = "pkcreated";
        float f1 = 1959.65f;
        Ofloat bean = getHome().create(pk, new Float(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).floatValue(), DELTA);
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjFloatRemove() throws Exception {
        String pk = "pktoremove";
        Ofloat bean = getHome().findByPrimaryKey(pk);
        Float f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
