/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.ointeger.Ointeger;
import org.objectweb.jonas.jtests.beans.etype.ointeger.OintegerHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Integer field of entity bean .
 * Beans used: etype/ointeger
 * @author Helene Joanin
 */
public abstract class A_EtypeOinteger extends JTestCase {

    public A_EtypeOinteger(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("ointeger", true);
    }

    /**
     * Return OintegerHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OintegerHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjIntegerFindByPk() throws Exception {
        String pk = "pk1";
        Ointeger bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjIntegerFindByF1() throws Exception {
        short f1 = (short) 5;
        Collection cBeans = getHome().findByF1(new Integer(f1));
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Ointeger bean = (Ointeger) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                             Ointeger.class);
            assertEquals("F1", f1, bean.getF1().shortValue());
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testObjIntegerGetF1() throws Exception {
        String pk = "pk2";
        Ointeger bean = getHome().findByPrimaryKey(pk);
        Integer f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", 2L, f1.shortValue());
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjIntegerGetF1Null() throws Exception {
        String pk = "pknull";
        Ointeger bean = getHome().findByPrimaryKey(pk);
        Integer f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjIntegerSetF1() throws Exception {
        String pk = "pk3";
        Ointeger bean = getHome().findByPrimaryKey(pk);
        short f1 = (short) 30;
        bean.setF1(new Integer(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).shortValue());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjIntegerSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Ointeger bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjIntegerCreate() throws Exception {
        String pk = "pkcreated";
        short f1 = (short) 1959;
        Ointeger bean = getHome().create(pk, new Integer(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).shortValue());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjIntegerRemove() throws Exception {
        String pk = "pktoremove";
        Ointeger bean = getHome().findByPrimaryKey(pk);
        Integer f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
