/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.olong.Olong;
import org.objectweb.jonas.jtests.beans.etype.olong.OlongHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Long field of entity bean .
 * Beans used: etype/olong
 * @author Helene Joanin
 */
public abstract class A_EtypeOlong extends JTestCase {

    public A_EtypeOlong(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("olong", true);
    }

    /**
     * Return OlongHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OlongHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjLongFindByPk() throws Exception {
        String pk = "pk1";
        Olong bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjLongFindByF1() throws Exception {
        long f1 = 5L;
        Collection cBeans = getHome().findByF1(new Long(f1));
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Olong bean = (Olong) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                       Olong.class);
            assertEquals("F1", f1, bean.getF1().longValue());
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testObjLongGetF1() throws Exception {
        String pk = "pk2";
        Olong bean = getHome().findByPrimaryKey(pk);
        Long f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", 2L, f1.longValue());
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjLongGetF1Null() throws Exception {
        String pk = "pknull";
        Olong bean = getHome().findByPrimaryKey(pk);
        Long f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjLongSetF1() throws Exception {
        String pk = "pk3";
        Olong bean = getHome().findByPrimaryKey(pk);
        long f1 = 30L;
        bean.setF1(new Long(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).longValue());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjLongSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Olong bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjLongCreate() throws Exception {
        String pk = "pkcreated";
        long f1 = 1959L;
        Olong bean = getHome().create(pk, new Long(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).longValue());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjLongRemove() throws Exception {
        String pk = "pktoremove";
        Olong bean = getHome().findByPrimaryKey(pk);
        Long f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
