/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.oshort.Oshort;
import org.objectweb.jonas.jtests.beans.etype.oshort.OshortHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Short field of entity bean .
 * Beans used: etype/oshort
 * @author Helene Joanin
 */
public abstract class A_EtypeOshort extends JTestCase {

    public A_EtypeOshort(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("oshort", true);
    }

    /**
     * Return OshortHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OshortHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testObjShortFindByPk() throws Exception {
        String pk = "pk1";
        Oshort bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testObjShortFindByF1() throws Exception {
        short f1 = (short) 5;
        Collection cBeans = getHome().findByF1(new Short(f1));
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Oshort bean = (Oshort) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                         Oshort.class);
            assertEquals("F1", f1, bean.getF1().shortValue());
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testObjShortGetF1() throws Exception {
        String pk = "pk2";
        Oshort bean = getHome().findByPrimaryKey(pk);
        Short f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", 2L, f1.shortValue());
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testObjShortGetF1Null() throws Exception {
        String pk = "pknull";
        Oshort bean = getHome().findByPrimaryKey(pk);
        Short f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testObjShortSetF1() throws Exception {
        String pk = "pk3";
        Oshort bean = getHome().findByPrimaryKey(pk);
        short f1 = (short) 30;
        bean.setF1(new Short(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).shortValue());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testObjShortSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Oshort bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testObjShortCreate() throws Exception {
        String pk = "pkcreated";
        short f1 = (short) 1959;
        Oshort bean = getHome().create(pk, new Short(f1));
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, (bean.getF1()).shortValue());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testObjShortRemove() throws Exception {
        String pk = "pktoremove";
        Oshort bean = getHome().findByPrimaryKey(pk);
        Short f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
