/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.sql.Date;
import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.osqldate.Osqldate;
import org.objectweb.jonas.jtests.beans.etype.osqldate.OsqldateHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.lang.Utildate field of entity bean .
 * Beans used: etype/osqldate
 * @author Helene Joanin
 */
public abstract class A_EtypeOsqldate extends JTestCase {

    public A_EtypeOsqldate(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("osqldate", true);
    }

    /**
     * Return OsqldateHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OsqldateHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testSqlDateFindByPk() throws Exception {
        String pk = "pk1";
        Osqldate bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testSqlDateFindByF1() throws Exception {
        Date d5 = Date.valueOf("1970-01-05");
        Collection cBeans = getHome().findByF1(d5);
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Osqldate bean = (Osqldate) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                             Osqldate.class);
            Date f1 = bean.getF1();
            assertTrue("F1", d5.equals(f1));
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testSqlDateGetF1() throws Exception {
        String pk = "pk2";
        Osqldate bean = getHome().findByPrimaryKey(pk);
        Date d2 = Date.valueOf("1970-01-02");
        Date f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", d2, f1);
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testSqlDateGetF1Null() throws Exception {
        String pk = "pknull";
        Osqldate bean = getHome().findByPrimaryKey(pk);
        Date f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testSqlDateSetF1() throws Exception {
        String pk = "pk3";
        Osqldate bean = getHome().findByPrimaryKey(pk);
        Date f1 = Date.valueOf("1970-01-30");
        bean.setF1(f1);
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, bean.getF1());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testSqlDateSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Osqldate bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testSqlDateCreate() throws Exception {
        String pk = "pkcreated";
        Date f1 = Date.valueOf("1975-05-13");
        Osqldate bean = getHome().create(pk, f1);
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, bean.getF1());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testSqlDateRemove() throws Exception {
        String pk = "pktoremove";
        Osqldate bean = getHome().findByPrimaryKey(pk);
        Date f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
