/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.sql.Time;
import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import org.objectweb.jonas.jtests.beans.etype.osqltime.Osqltime;
import org.objectweb.jonas.jtests.beans.etype.osqltime.OsqltimeHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about java.sql.Time field of entity bean .
 * Beans used: etype/osqltime
 * @author Helene Joanin
 */
public abstract class A_EtypeOsqltime extends JTestCase {

    public A_EtypeOsqltime(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("osqltime", true);
    }

    /**
     * Return OsqltimeHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public OsqltimeHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testSqlTimeFindByPk() throws Exception {
        String pk = "pk1";
        Osqltime bean = getHome().findByPrimaryKey(pk);
        assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testSqlTimeFindByF1() throws Exception {
        Time d5 = Time.valueOf("05:00:00");
        Collection cBeans = getHome().findByF1(d5);
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Osqltime bean = (Osqltime) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                             Osqltime.class);
            Time f1 = bean.getF1();
            assertEquals("F1", d5, f1);
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testSqlTimeGetF1() throws Exception {
        String pk = "pk2";
        Osqltime bean = getHome().findByPrimaryKey(pk);
        Time d2 = Time.valueOf("02:00:00");
        Time f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", Time.valueOf("02:00:00"), f1);
    }

    /**
     * getF1Null() test: f1 with a 'null' value
     */
    public void testSqlTimeGetF1Null() throws Exception {
        String pk = "pknull";
        Osqltime bean = getHome().findByPrimaryKey(pk);
        Time f1 = bean.getF1();
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1 not null", f1);
    }

    /**
     * setF1() test
     */
    public void testSqlTimeSetF1() throws Exception {
        String pk = "pk3";
        Osqltime bean = getHome().findByPrimaryKey(pk);
        Time f1 = Time.valueOf("06:00:00");
        bean.setF1(f1);
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, bean.getF1());
    }

    /**
     * setF1Null() test: f1 with a 'null' value
     */
    public void testSqlTimeSetF1Null() throws Exception {
        String pk = "pkchangenull";
        Osqltime bean = getHome().findByPrimaryKey(pk);
        bean.setF1(null);
        assertEquals("Pk", pk, bean.getPk());
        assertNull("F1", bean.getF1());
    }

    /**
     * create() test
     */
    public void testSqlTimeCreate() throws Exception {
        String pk = "pkcreated";
        Time f1 = Time.valueOf("19:00:00");
        Osqltime bean = getHome().create(pk, f1);
        assertEquals("Pk", pk, bean.getPk());
        assertEquals("F1", f1, bean.getF1());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testSqlTimeRemove() throws Exception {
        String pk = "pktoremove";
        Osqltime bean = getHome().findByPrimaryKey(pk);
        Time f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
