/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.Collection;
import java.util.Iterator;
import javax.ejb.FinderException;

import junit.framework.Assert;

import org.objectweb.jonas.jtests.beans.etype.plong.Plong;
import org.objectweb.jonas.jtests.beans.etype.plong.PlongHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This set of test are common to CMP version 1 and CMP version 2
 * These are tests about 'long' field of entity bean .
 * Beans used: etype/plong
 * @author Helene Joanin
 */
public abstract class A_EtypePlong extends JTestCase {

    public A_EtypePlong(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        useBeans("plong", true);
    }

    /**
     * Return PlongHome, that can be either CMP version 1 or CMP version 2 bean.
     */
    abstract public PlongHome getHome();

    /**
     * findByPrimaryKey() test
     */
    public void testPrimLongFindByPk() throws Exception {
        String pk = "pk1";
        Plong bean = getHome().findByPrimaryKey(pk);
        Assert.assertEquals("Pk", pk, bean.getPk());
    }

    /**
     * findByF1() test
     */
    public void testPrimLongFindByF1() throws Exception {
        long f1 = 5L;
        Collection cBeans = getHome().findByF1(f1);
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Plong bean = (Plong) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                       Plong.class);
            Assert.assertEquals("F1", f1, bean.getF1());
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * findByF1Literal5() test
     */
    public void testPrimLongFindByF1Literal5() throws Exception {
        long f1 = 5L;
        Collection cBeans = getHome().findByF1Literal5();
        int nb = 0;
        Iterator iBeans = cBeans.iterator();
        while (iBeans.hasNext()) {
            Plong bean = (Plong) javax.rmi.PortableRemoteObject.narrow(iBeans.next(),
                                                                       Plong.class);
            Assert.assertEquals("F1", f1, bean.getF1());
            nb++;
        }
        assertEquals("Beans number: ", 2, nb);
    }

    /**
     * getF1() test
     */
    public void testPrimLongGetF1() throws Exception {
        String pk = "pk2";
        Plong bean = getHome().findByPrimaryKey(pk);
        long f1 = bean.getF1();
        Assert.assertEquals("Pk", pk, bean.getPk());
        Assert.assertEquals("F1", 2L, f1);
    }

    /**
     * setF1() test
     */
    public void testPrimLongSetF1() throws Exception {
        String pk = "pk3";
        Plong bean = getHome().findByPrimaryKey(pk);
        long f1 = 30L;
        bean.setF1(f1);
        Assert.assertEquals("Pk", pk, bean.getPk());
        Assert.assertEquals("F1", f1, bean.getF1());
    }

    /**
     * create() test
     */
    public void testPrimLongCreate() throws Exception {
        String pk = "pkcreated";
        long f1 = 1959L;
        Plong bean = getHome().create(pk, f1);
        Assert.assertEquals("Pk", pk, bean.getPk());
        Assert.assertEquals("F1", f1, bean.getF1());
        bean = getHome().findByPrimaryKey(pk);
        // cleaning
        bean.remove();
    }

    /**
     * remove() test
     */
    public void testPrimLongRemove() throws Exception {
        String pk = "pktoremove";
        Plong bean = getHome().findByPrimaryKey(pk);
        long f1 = bean.getF1();
        bean.remove();
        try {
            getHome().findByPrimaryKey(pk);
            fail("not removed");
        } catch (FinderException e) {
            // ok
        }
        // cleaning
        getHome().create(pk, f1);
    }

}
