/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This is a test suite about the different cases of the types of the entity bean's fields.
 * It run suites for CMP version 1, and for CMP version 2.
 * Beans used: etype/*
 * @author Helene Joanin
 */

public class C_EtypeEC extends JTestCase {

    public C_EtypeEC(String name) {
        super(name);
    }

    public static Test suite() {

        TestSuite suite = new TestSuite();

        // primitive double
        suite.addTest(F_EtypePdoubleEC.suite());
        // java.lang.Double
        suite.addTest(F_EtypeOdoubleEC.suite());

        // primitive float
        suite.addTest(F_EtypePfloatEC.suite());
        // java.lang.Float
        suite.addTest(F_EtypeOfloatEC.suite());

        // primitive long
        suite.addTest(F_EtypePlongEC.suite());
        // java.lang.Long
        suite.addTest(F_EtypeOlongEC.suite());

        // primitive short
        suite.addTest(F_EtypePshortEC.suite());
        // java.lang.Short
        suite.addTest(F_EtypeOshortEC.suite());

        // java.lang.Byte
        suite.addTest(F_EtypeObyteEC.suite());

        // java.lang.Integer
        suite.addTest(F_EtypeOintegerEC.suite());

        // java.sql.Date
        suite.addTest(F_EtypeOsqldateEC.suite());
        // java.sql.Time
        suite.addTest(F_EtypeOsqltimeEC.suite());
        // java.sql.Timestamp
        suite.addTest(F_EtypeOsqltsEC.suite());

        // primitive boolean
        suite.addTest(F_EtypePbooleanEC.suite());
        // java.lang.Boolean
        suite.addTest(F_EtypeObooleanEC.suite());

        // serializable type
        suite.addTest(F_EtypeOserEC.suite());

        // java.math.BigDecimal
        suite.addTest(F_EtypeObigdecimalEC.suite());

        // byte[]
        suite.addTest(F_EtypeByteArrayEC.suite());

        return suite;

    }

    public static void main (String args[]) {
        junit.textui.TestRunner.run(suite());
    }
}
