/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This is a test suite about the different cases of the types of the entity bean's fields.
 * It run suites for CMP version 2.
 * Beans used: etype/*
 * @author Helene Joanin
 */

public class C_EtypeEC2 extends JTestCase {

    public C_EtypeEC2(String name) {
        super(name);
    }

    public static Test suite() {

        TestSuite suite = new TestSuite();

        // primitive double
        suite.addTest(F_EtypePdoubleEC2.suite());
        // java.lang.Double
        suite.addTest(F_EtypeOdoubleEC2.suite());

        // primitive float
        suite.addTest(F_EtypePfloatEC2.suite());
        // java.lang.Float
        suite.addTest(F_EtypeOfloatEC2.suite());
        // java.lang.Float for the pk field
        suite.addTest(F_EtypeOfloat4pkEC2.suite());

        // primitive long
        suite.addTest(F_EtypePlongEC2.suite());
        // java.lang.Long
        suite.addTest(F_EtypeOlongEC2.suite());

        // primitive short
        suite.addTest(F_EtypePshortEC2.suite());
        // java.lang.Short
        suite.addTest(F_EtypeOshortEC2.suite());

        // java.lang.Byte
        suite.addTest(F_EtypeObyteEC2.suite());

        // java.lang.Integer
        suite.addTest(F_EtypeOintegerEC2.suite());

        // java.util.Date (CMP version 2 only)
        suite.addTest(F_EtypeOutildateEC2.suite());

        // java.sql.Date
        suite.addTest(F_EtypeOsqldateEC2.suite());
        // java.sql.Time
        suite.addTest(F_EtypeOsqltimeEC2.suite());
        // java.sql.Timestamp
        suite.addTest(F_EtypeOsqltsEC2.suite());
        // java.sql.Timestamp for pk
        suite.addTest(F_EtypeOsqlts4pkEC2.suite());

        // primitive boolean
        suite.addTest(F_EtypePbooleanEC2.suite());
        // java.lang.Boolean
        suite.addTest(F_EtypeObooleanEC2.suite());

        // serializable type
        suite.addTest(F_EtypeOserEC2.suite());

        // java.math.BigDecimal
        suite.addTest(F_EtypeObigdecimalEC2.suite());

        // byte[]
        suite.addTest(F_EtypeByteArrayEC2.suite());

        return suite;

    }

    public static void main (String args[]) {
        junit.textui.TestRunner.run(suite());
    }
}
