/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.annuaire.PersonneHome;

/**
 * This is an advanced test suite for home interface on entity bean CMP v2.
 * * All tests common to CMP v1 and CMP v2 are in the inherited class A_AdvancedHomeEC.
 * Beans used: annuaire
 * @author Philippe Coq, Philippe Durieux, Helene Joanin (jonas team)
 */
public class F_AdvancedHomeEC2 extends A_AdvancedHomeEC {

    private static String BEAN_HOME = "annuairePersonneEC2Home";
    private static PersonneHome ehome = null;

    public F_AdvancedHomeEC2(String name) {
        super(name);
    }

    protected boolean isInit = false;

    protected void setUp() {
        super.setUp();
        if (!isInit) {
            useBeans("annuaire", false);
            getHome();
            isInit = true;
        }
    }

    public PersonneHome getHome() {
        if (ehome == null) {
            try {
                ehome = (PersonneHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), PersonneHome.class);
            } catch (NamingException e) {
                fail("Cannot get bean home");
            }
            // check if tables have been initialized
            try {
                ehome.findByPrimaryKey("Charly Mingus");
            } catch (Exception e) {
            // init here tables if CMP2
            try {
                    utx.begin();
                ehome.create("Philippe Durieux", "638");
                ehome.create("Philippe Coq", "1235456");
                ehome.create("Adriana Danes", "1233456");
                ehome.create("Helene Joanin", "1230456");
                ehome.create("Gerard Vandome", "1232456");
                ehome.create("Francois Exertier", "1323456");
                ehome.create("Emmanuel Facarde", "1234356");
                ehome.create("Charly Mingus", "1238456");
                ehome.create("Thelonious Monk", "1239456");
                ehome.create("Jean-Luc Richard", "1203456");
            } catch (Exception i) {
                debug("Cannote create bean" + i);
                } finally {
                    try {
                        utx.commit();
                    } catch (Exception e3) {
                    }
                }
            }
        }
        return ehome;
    }

    public static Test suite() {
        return new TestSuite(F_AdvancedHomeEC2.class);
    }

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_AdvancedHomeEC2(testtorun));
        }
    }
}
