/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.relation.mnu.AHomeRemote;
import org.objectweb.jonas.jtests.beans.relation.mnu.ARemote;
import org.objectweb.jonas.jtests.beans.relation.mnu.BHomeRemote;
import org.objectweb.jonas.jtests.beans.relation.mnu.BRemote;

/**
 * For testing many-to-many unidirectional relationships
 * @author Ph Durieux
 **/
public class F_mnu_CRW_EC2 extends A_mnu {

    private static String BEAN_HOME_A = "relation_mnu_AtxHome";
    protected static AHomeRemote ahome = null;
    private static String BEAN_HOME_B = "relation_mnu_BtxHome";
    protected static BHomeRemote bhome = null;


    public F_mnu_CRW_EC2(String name) {
        super(name);
    }

    public AHomeRemote getAHome() {
        if (ahome == null) {
            try {
                ahome = (AHomeRemote) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_A),
                        AHomeRemote.class);
            } catch (NamingException e) {
                fail("Cannot get bean home: " + e.getMessage());
            }
        }
        return ahome;
    }

    public BHomeRemote getBHome() {
        if (bhome == null) {
            try {
                bhome = (BHomeRemote) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME_B),
                        BHomeRemote.class);
            } catch (NamingException e) {
                fail("Cannot get bean home: " + e.getMessage());
            }
        }
        return bhome;
    }

    public void testRollback() throws Exception {
        ARemote a = getAHome().findByPrimaryKey("a1");
        Collection orig = a.retrieveB();
        int colsz = orig.size();

        // disassociate and commit
        utx.begin();
        a.assignB(new ArrayList());
        utx.commit();
        assertEquals("cmr not reset to null", 0, a.retrieveB().size());

        // revert to original state outside of a user transaction
        a.assignB(orig);

        // sync(false) here forces test to pass

        // retrieveB inside a transaction
        utx.begin();
        assertEquals("CMR not reset to original value", colsz, a.retrieveB().size());
        utx.rollback();

        // verify that retrieveB returns correct value after rollback
        try {
            assertEquals("rollback should not modify this value", colsz, a.retrieveB().size());
        } finally {
            // force sync to cleanup
            sync(true);
        }
        checkIsInitialState();
    }

    public static Test suite() {
        return new TestSuite(F_mnu_CRW_EC2.class);
    }

    public static void main(String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_mnu_CRW_EC2(testtorun));
        }
    }

}
