/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.j2eeca;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.transaction.UserTransaction;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.j2eeca.transactedCA;
import org.objectweb.jonas.jtests.beans.j2eeca.transactedCAHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * F_xatransactionTests tests JOnAS Connection Architecture XA Transaction
 */

public class F_xatransactionTest extends JTestCase {

    private static UserTransaction utx = null;

    static Context ctx = null;

    // Lookup bean home

    protected static String BEAN_HOME = "transactedCAHome";
    protected static transactedCAHome home = null;
    private static final String RAR_JNDI_NAME = "FictionalXATransaction";
    final public int CLOSE_HANDLE = 0;
    final public int CLOSE_PHYSICAL = 1;
    public int CloseType = 0;
    static transactedCA bean = null;
    public String UseBeans="j2eeca";
    
    public F_xatransactionTest(String name) {
        super(name);
    }

    protected void setUp() {
        super.setUp();
        try {
    	    // get JNDI initial context

            if (ctx == null) {
	            ctx = new InitialContext();
            }

	        // We want to start transactions from client: get UserTransaction

            if (utx == null) {
	            utx = (UserTransaction) ctx.lookup("javax.transaction.UserTransaction");
            }

            if (home == null) {
    	        useBeans(UseBeans, false);
            }
            getBean();
            assertTrue (6 == 6);

	    } catch (Exception e) {
	        fail("Cannot lookup UserTransaction in setUp: " +e);
        }
    }
    private void getBean() throws Exception {
        // Connecting to transactedCAHome thru JNDI

        if (home == null) {
	        home = (transactedCAHome)PortableRemoteObject.narrow(
							    ctx.lookup(BEAN_HOME),
							    transactedCAHome.class);
            bean = home.create();
        }
    }
    protected void tearDown() throws Exception {
        //bean.closeUp(CloseType);
    }
    protected void startUp(String testName) {
	    try {
	        bean.method1(RAR_JNDI_NAME, testName);
	    } catch (Exception ee) {
	        ee.printStackTrace();
	        System.exit(2);
	    }
    }

// test list ****************************************************

    /**
     * The application server invokes the ManagedConnection.getXAResource method
     * to get the XAResource instance associated with the ManagedConnection instance.
     * Then, application server calls the TM, which in turn calls the 
     * XAResourceImpl.start() method.
     * 
     * This test verifies the existence of the instance, assigned Xid, and start()
     */
    public void testIsXaresource() {
        CloseType=CLOSE_PHYSICAL;
        
        try {
            utx.begin();
            startUp("testIsXaresource");
            String ans = bean.getXid();
            assertEquals("OK", ans);
            bean.closeUp(CloseType);
            utx.commit();
            assertTrue(5 == 5);
        } catch (Exception e) {
            try {
                utx.rollback();
            } catch (Exception f) {
                assertTrue(5 == 5);
            }
            assertTrue(5 == 5);
        }
    }
    /**
     *  At connection close, the application server performs "transactional" cleanup.
     *  The application server dissociates the XAResource instance (corresponding 
     *  to the ManagedConnection "mc" instance) from the transaction manager.
     *  The transaction manager calls XAResource.end(Xid, flag)
     * <p>
     *  The ConnectionEvent is CLOSE_CONNECTION. This test verifies that application 
     *  server called TM which in turn called XAResource.end(). 
     *  
     *
     */
    public void testIsXAend1() {
        CloseType=CLOSE_HANDLE; // leave physical connection open
        
        try {
            utx.begin();
            startUp("testIsXAend1");
            assertTrue(bean.getCMInstance());  // is physical connection o.k.
            bean.closeUp(CloseType);
            assertTrue(bean.getCMInstance());  // should still be o.k.
            utx.commit();
            assertTrue(5 == 5);
        } catch (Exception e) {
            try {
                utx.rollback();
            } catch (Exception f) {
                assertTrue(5 == 5);
            }
            assertTrue(5 == 5);
        }
    }
    /**
     *  At connection close, the application server performs "transactional" cleanup.
     *  The application server dissociates the XAResource instance (corresponding 
     *  to the ManagedConnection "mc" instance) from the transaction manager.
     *  The transaction manager calls XAResource.end(Xid, flag)
     * <p>
     *  The ConnectionEvent is CONNECTION_ERROR_OCCURRED to force a physical close.
     *  verifies mc.XAR reference is null
     *
     */
    public void testIsXAend2() {
        CloseType=CLOSE_PHYSICAL;
        
        try {
            utx.begin();
            startUp("testIsXAend2");
            assertTrue(bean.getCMInstance());  // is physical connection o.k.
            bean.closeUp(CloseType);
            String ans = bean.getXid();
            //System.out.print(" -testIsXAend2 does not work. end() not called by TM-");
            //System.out.println("");
            assertEquals("FAIL", ans);        // XAR is dissociated
            utx.commit();
            assertTrue(5 == 5);
        } catch (Exception e) {
            try {
                utx.rollback();
            } catch (Exception f) {
                assertTrue(5 == 5);
            }
            assertFalse(5 == 5);
        }
    }

    public static Test suite() {
        return new TestSuite(F_xatransactionTest.class);
    }

    public static void main (String args[]) {

        String testtorun = null;

        // Get args

        for (int argn = 0; argn < args.length; argn++) {

            String s_arg = args[argn];
            Integer i_arg;

            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }

        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_xatransactionTest(testtorun));
        }
    }
}
