/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.timer;


import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.folder.Folder;
import org.objectweb.jonas.jtests.beans.folder.FolderHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * This is test of the TimerService.
 * @author Philippe Durieux (jonas team)
 */
public class F_TimerFromSF extends JTestCase {

    protected static FolderHome fhome = null;
    protected Folder folder = null;
    
    /**
     * constructor
     * @param name name of the test suite.
     */
    public F_TimerFromSF(String name) {
        super(name);
    }

    /**
     * init environment:
     * - load beans
     * - get the home for session beans
     */
    protected void setUp() {
        super.setUp();
        if (fhome == null) {
            useBeans("folder", true);
            try {
                fhome = (FolderHome) PortableRemoteObject.narrow(ictx.lookup("FolderSYHome"), FolderHome.class);
                assertNotNull(fhome);
            } catch (NamingException e) {
                fail("Cannot get bean home");
            }
        }
        if (folder == null) {
            try {
                folder = fhome.create();
                assertNotNull(folder);
            } catch (Exception e) {
                fail("Cannot create folder session " + e);
            }
        }
    }

    public void testAccessRemovedTimer() throws Exception {
        boolean ok = folder.testTimerRemoved("timerfile");
        assertTrue("Timer Still Accessed after bean remove", ok);
    }
    
    public void testAccessCanceledTimer() throws Exception {
        boolean ok = folder.testTimerCancel("timerfile2");
        assertTrue("Timer Still Accessed after cancel", ok);
    }
    
    public static Test suite() {
        return new TestSuite(F_TimerFromSF.class);
    }

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String sarg = args[argn];
            if (sarg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_TimerFromSF(testtorun));
        }
    }
}

