/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.timer;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.transacted.Simple;
import org.objectweb.jonas.jtests.beans.transacted.SimpleSHome;

/**
 * test TimerService on Stateless Session Beans
 */
public class F_TimerSL extends A_TimerSession {

    private static String BEAN_HOME = "transactedSimpleSLHome";
    protected static SimpleSHome home = null;

    /**
     * constructor
     * @param name name of the test suite.
     */
    public F_TimerSL(String name) {
        super(name);
    }

    /**
     * Create a bean and return it.
     */
    public Simple getSimple(int i) throws Exception {
        return home.create();
    }

    /**
     * init environment:
     * - load beans
     * - get the home for session beans
     * - cancel timers left over from previous tests
     */
    protected void setUp() {
        super.setUp();
        if (home == null) {
            try {
                home = (SimpleSHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), SimpleSHome.class);
            } catch (NamingException e) {
                fail("Cannot get Home:" + e);
            }
        }
    }

    /**
     * - cancel timers from test
     */
    protected void tearDown() throws Exception {
        // Remove this line to test the persistent timers !
        cancelSessionTimers();
        super.tearDown();
     }

    private void cancelSessionTimers() {
        Simple s;
        try {
            s = getSimple(0);
        } catch (Exception e) {
            throw new RuntimeException("could not create session bean", e);
        }

        try {
            s.cancelTimers();
            s.remove();
        } catch (java.rmi.RemoteException re) {
            throw new RuntimeException("could not cancel session timers", re);
        } catch (javax.ejb.RemoveException re) {
            throw new RuntimeException("could not remove session bean", re);
        }
    }

    /**
     * This suite is all Session Statless test cases
     */
    public static Test suite() {
        return new TestSuite(F_TimerSL.class);
    }

    public static void main (String args[]) {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String sarg = args[argn];
            if (sarg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_TimerSL(testtorun));
        }
    }
}
