/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.tables;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.Statement;

import javax.naming.NamingException;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Class to create tables for the 'mou' bean.
 * @author Philippe Durieux
 */
public class Tmou extends Tmanager {

    /**
     * Entry point
     */
    public static void init() throws NamingException, RemoteException {
        mgrInit();
        createTableA();
        createTableB();
    }

    /**
     * create a table for the bean A
     */
    private static void createTableA() throws RemoteException {

        String name = "JT2_MOU_A1";

        // get connection
        Connection conn = null;
        try {
            conn = dataSource.getConnection();
        } catch(Exception e) {
            throw new RemoteException("Cannot get Connection");
        }
	
        Statement stmt;
        try {
            stmt = conn.createStatement();
            stmt.execute("DROP TABLE " + name);
            stmt.close();
            logger.log(BasicLevel.INFO, "Table "+name+" dropped");
        } catch(Exception e) {
            logger.log(BasicLevel.DEBUG, "Exception in dropTable : \n"+e);
        }
        try {
            stmt = conn.createStatement();
            stmt.execute("create table " + name + 
                         "( c_ida varchar(30) not null primary key, cfk_idb varchar(30) not null )");
            stmt.close();
            conn.close();		// release connection
        } catch(Exception e) {
            logger.log(BasicLevel.ERROR, "Exception in createTable : "+e);
            throw new RemoteException("Exception in createTable : "+e);
        }
        logger.log(BasicLevel.INFO, "Table "+name+" created");
    }

    /**
     * create a table for the bean B
     */
    private static void createTableB() throws RemoteException {

        String name = "JT2_MOU_B1";

        // get connection
        Connection conn = null;
        try {
            conn = dataSource.getConnection();
        } catch(Exception e) {
            throw new RemoteException("Cannot get Connection");
        }
	
        Statement stmt;
        try {
            stmt = conn.createStatement();
            stmt.execute("DROP TABLE " + name);
            stmt.close();
            logger.log(BasicLevel.INFO, "Table "+name+" dropped");
        } catch(Exception e) {
            logger.log(BasicLevel.DEBUG, "Exception in dropTable : \n"+e);
        }
        try {
            stmt = conn.createStatement();
            stmt.execute("create table " + name + 
                         "( c_idb varchar(30) not null primary key )");
            stmt.close();
            conn.close();		// release connection
        } catch(Exception e) {
            logger.log(BasicLevel.ERROR, "Exception in createTable : "+e);
            throw new RemoteException("Exception in createTable : "+e);
        }
        logger.log(BasicLevel.INFO, "Table "+name+" created");
    }

}
