/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.tables;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.Statement;

import javax.naming.NamingException;

import org.objectweb.jonas.jtests.util.Env;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Class to create tables for the 'pboolean' bean of 'etype'.
 * @author Helene Joanin 
 */
public class Tpboolean extends Tmanager {

    /**
     * Entry point
     */
    public static void init() throws NamingException, RemoteException {
	mgrInit();
	createTable("JT_EtypePbooleanEC");
    }

    /**
     * create a table for the pboolean bean of etype
     */
    private static void createTable(String name) throws RemoteException {

	// get connection
	Connection conn = null;
	try {
	    conn = dataSource.getConnection();
	} catch(Exception e) {
	    throw new RemoteException("Cannot get Connection");
	}
	
	Statement stmt;
	try {
	    stmt = conn.createStatement();
	    stmt.execute("DROP TABLE "+name);
	    stmt.close();
	    logger.log(BasicLevel.INFO, "Table "+name+" dropped");
	} catch(Exception e) {
	    logger.log(BasicLevel.DEBUG, "Exception in dropTable : \n"+e);
	}
	try {
	    int dbType = Env.getDatabaseType(conn);
	    String cTypeName;
	    switch (dbType) {
	    case Env.DB_ORACLE: cTypeName = "number"; break;
	    case Env.DB_POSGRESQL: cTypeName = "boolean"; break;
	    default: cTypeName = "bit";	// standard jdbc type name
	    }
	    stmt = conn.createStatement();
	    stmt.execute("create table " + name + 
			 "( c_pk varchar(30) not null primary key, c_f1 "+cTypeName+")");
        if ((dbType == Env.DB_ORACLE) || (dbType == Env.DB_MYSQL)) {
            stmt.execute("insert into "+name+" values('pk1', 0)");
            stmt.execute("insert into "+name+" values('pk2', 1)");
            stmt.execute("insert into "+name+" values('pk3', 0)");
            stmt.execute("insert into "+name+" values('pk4', 1)");
            stmt.execute("insert into "+name+" values('pk5', 0)");
            stmt.execute("insert into "+name+" values('pktoremove', 1)");
        } else {
            stmt.execute("insert into "+name+" values('pk1', 'false')");
            stmt.execute("insert into "+name+" values('pk2', 'true')");
            stmt.execute("insert into "+name+" values('pk3', 'false')");
            stmt.execute("insert into "+name+" values('pk4', 'true')");
            stmt.execute("insert into "+name+" values('pk5', 'false')");
            stmt.execute("insert into "+name+" values('pktoremove', 'true')");
        }

	    stmt.close();
	    conn.close();		// release connection
	} catch(Exception e) {
	    logger.log(BasicLevel.ERROR, "Exception in createTable : "+e);
	    throw new RemoteException("Exception in createTable : "+e);
	}
	logger.log(BasicLevel.INFO, "Table "+name+" created");
    }

}
