/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2005 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or 1any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.examples.clients.webservices;

import junit.framework.TestSuite;

import com.meterware.httpunit.WebResponse;


/**
 * Define a class to test the webServices examples wsAccessTest
 * Test WebServices deployment OK
 * @author Guillaume Sauthier
 */
public class F_WsAccessTest extends A_WebServices {

    /**
     * URL of the wsAccessTest page
     */
    private static final String URL_WSACCESSTEST = "/wsaccess/";

    /**
     * Main method
     * @param args the arguments
     */
    public static void main(String[] args) {

        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String sArg = args[argn];
            if (sArg.equals("-n")) {
                testtorun = args[++argn];
            }
        }

        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else {
            junit.textui.TestRunner.run(new F_WsAccessTest(testtorun));
        }
    }

    /**
     * Get a new TestSuite for this class
     * @return a new TestSuite for this class
     */
    public static TestSuite suite() {
        return new TestSuite(F_WsAccessTest.class);
    }

    /**
     * Setup need for these tests
     * earsample is required
     * @throws Exception if it fails
     */
    protected void setUp() throws Exception {
        super.setUp();
        useWar("wsaccess");
    }

    /**
     * Constructor with a specified name
     * @param s name
     */
    public F_WsAccessTest(String s) {
        super(s, URL_WSACCESSTEST);
    }

    /**
     * Check if home page is accessible
     * @throws Exception if an error occurs
     */
    public void testDeploymentCompleted() throws Exception {
        // 2 links + 3 logo links on homepage
        checkDeploymentCompleted(2+3);
    }

    /**
     * Check if JSP can access external google WebService
     * @throws Exception if an error occurs
     */
    public void testWebServiceJspAccess() throws Exception {

        WebResponse wr = executeGoogleWebService("googleByJSP.jsp",
                                                 "ObjectWeb JOnAS");
        assertTrue("a list is returned", wr.getLinks().length > 0);
    }

    /**
     * Check if JSP can access external google WebService
     * @throws Exception if an error occurs
     */
    public void testWebServiceServletAccess() throws Exception {

        WebResponse wr = executeGoogleWebService("googleByServlet.html",
                                                 "ObjectWeb JOnAS");
        assertTrue("a list is returned", wr.getLinks().length > 0);

    }
}
