// OrderHome.java

package org.objectweb.jonas.stests.appli;

import java.rmi.RemoteException;
import java.util.Vector;

import javax.ejb.CreateException;
import javax.ejb.EJBHome;
import javax.ejb.FinderException;

/**
 * Home interface for the bean Order
 */
public interface OrderHome extends EJBHome {

	/**
     * Create a new Order.
	 * <p>
     * @param String warehouseId The id of the warehouse
     * @param int districtId The id of the district
     * @param float orderId The order id
     * @param Vector orderDetails The orderDetails for the order
     * @return Order
     * @exception javax.ejb.CreateException Creation Exception
     * @exception java.rmi.RemoteException Remote Exception
	 */

    public Order create(String warehouseId, 
                        int districtId, 
                        Integer customerId, 
                        float orderId, Vector orderDetails)
        throws CreateException, RemoteException;


	/**
     * Retrieve an individual order by the OrderID.
	 * <p>
     * @param OrderID primaryKey 
     * @return Order
     * @exception javax.ejb.FinderException Find Exception
     * @exception java.rmi.RemoteException Remote Exception
	 */

    public Order findByPrimaryKey(OrderID primaryKey)
        throws FinderException, RemoteException;
	

	/**
     * Retrieve an individual order given the warehouse, district and
     * order ids as Strings
	 * <p>
     * @param String wID warehouse id 
     * @param String dID warehouse id 
     * @param String oID warehouse id 
     * @return Order
     * @exception javax.ejb.FinderException Find Exception
     * @exception java.rmi.RemoteException Remote Exception
	 */

	public Order findByID(String wID, String dID, String oID)
        throws FinderException, RemoteException;

}
