// Stock.java

package org.objectweb.jonas.stests.appli;

import java.rmi.RemoteException;
import javax.ejb.EJBObject;

/**
 * Stock remote interface
 */
public interface Stock extends EJBObject {
    /**
     * Get the warehouse id.
     * <p>
     * @return String
     * @exception java.rmi.RemoteException Remote exception
     */

    public String getWarehouseID() throws RemoteException;
    public Integer getItemID() throws RemoteException;
 
    /**
     * Get the Stock Quantity 
     * <p>
     * @return integer
     * @exception java.rmi.RemoteException Remote exception
     */
 
    public int getStockQuantity() throws RemoteException;
 
    /**
     * Increase the stock quantity   by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */
    public void increaseStockQuantity(int stockQty) throws RemoteException;
    /**
     * Decrease the stock quantity   by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */
    public void decreaseStockQuantity(int stockQty) throws RemoteException;

 
    /**
     * Get the Year to Date Quantity 
     * <p>
     * @param none <i>(Mandatory)</i>
     * @return integer
     * @exception java.rmi.RemoteException Remote exception
     */
 
    public int getYearToDate() throws RemoteException;

    /**
     * Increase the Year to date quantity   by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */
    public void increaseYearToDate(int qty) throws RemoteException;
 
    /**
     * Decrease the Year to date quantity   by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */
 
    public void decreaseYearToDate(int qty) throws RemoteException;

    /**
     * Get the Order Quantity 
     * <p>
     * @param none <i>(Mandatory)</i>
     * @return integer
     * @exception java.rmi.RemoteException Remote exception
     */
 
    public int getOrderQuantity() throws RemoteException;
 
    /**
     * Increase the Order Quantity by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */
 
    public void increaseOrderQuantity(int qty) throws RemoteException;
 
    /**
     * Decrease the Order Quantity by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */

    public void decreaseOrderQuantity(int qty) throws RemoteException;

    /**
     * Get the Remote Order Quantity 
     * <p>
     * @param none <i>(Mandatory)</i>
     * @return integer
     * @exception java.rmi.RemoteException Remote exception
     */
 
    public int getRemOrderQuantity() throws RemoteException;

    /** Increase the Remote Order Quantity by a quantity
        * <p>
        * @param integer quantity <i>(Mandatory)</i>
        * @return void
        * @exception java.rmi.RemoteException Remote exception
        */
 
        public void increaseRemOrderQuantity(int qty) throws RemoteException;

    /**
     * Decrease the Remote Order Quantity by a quantity
     * <p>
     * @param integer quantity <i>(Mandatory)</i>
     * @return void
     * @exception java.rmi.RemoteException Remote exception
     */

    public void decreaseRemOrderQuantity(int qty) throws RemoteException;
 
}
